

from FL import *

"""
	
	
	Measure the horizontal and vertical sections the glyph.
		
	Tool script for FontLab Studio 5
	Requires up tp date RoboFab.
	
	Installing:
		Place this script in the FontLab Studio 5/Macros/System folder. This is a special scripts
		folder for tools which will end up in the tools menu, not the Macros folder.
		
		Make sure this folder and its contents have sufficient permissions to read, write and execute.

		On Mac it can be here:
		/Library/Application Support/FontLab/Studio 5/Macros/System
		
	Using:
		Open a glyph window. Select the the tool, select the cursor.
		Normal clicking will draw a horizontal slice at the click position.
		Command-Click will draw a vertical slice at the click position.
		
	Not tested on Windows. I anticipate installing will be possible, but at  a different location.
	The command-click event handling might be different on Windows.
	
	
	RoboFab License Agreement (BSD License)

	Copyright (c) 2003-2007, Erik van Blokland / erik@letterror.com

	All rights reserved.

	Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

	Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
	Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
	Neither the name of the RoboFab Developers nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""

from robofab.world import CurrentGlyph, CurrentFont
from robofab.pens.marginPen import MarginPen
import math

class Tool:
	def __init__(self):
		self.mouseDown = False
		self.currentKeys = None
		self.contourMargins = None
		self.allMargins = None
		self.glyphMargins = None
		self.sampleHeight = None
		self.isHorizontal = True
		self.modKey = None
		self.colors = [0x157a92, 0x157a92, 0x265a81, 0x324464, 0x1b451d, 0x3a6c22, 0x875386, 0xaf397c, 0xb54132, 0x109ee2]
		self.keyCallbacks = {0:"mod_noKey", 256:"mod_cmd", 512:"mod_shift", 768:"mod_1", 2048:'mod_option'}
		
		self.labelOffset = [0, -1, 1, -2, 2, -3, 3, -4, 4]
		self.labelOffsetIndex = 0
		
		# remove crud
		g = CurrentGlyph()
		g.update()
		self.currentGlyphName = g.name
		
	
	def _getColor(self, index):
		index = index%len(self.colors)
		return self.colors[index]
	
	def drawHorizontalSection(self, canvas, color, start, end, height, value, direction=1):
		"""Draw the horizontal section measurements with cutesy curves and colors."""
		s = min(start, end)
		e = max(start, end)
		start = s
		end = e
		k = (round(start, 2), round(end, 2))
		if k in self._done:
			return
		self._done.append(k)
		if direction == 1:
			canvas.pen_style = cPEN_SOLID
			extraTextShift = 0
		else:
			canvas.pen_style = cPEN_DOT
			extraTextShift = -10
		canvas.brush_color = color
		p1 = fl.GlyphToScreen(Point(start, height))
		p2 = fl.GlyphToScreen(Point(end, height))
		yOffset = int((end-start)*.1)
		d = max(p1.x, p2.x) - min(p1.x, p2.x)
		bcp1 = Point(int(p1.x+.33*d), p1.y+yOffset)
		bcp2 = Point(int(p1.x+.66*d), p1.y+yOffset)
		center = Point((p1.x+p2.x)*0.5 - 10, p1.y + 0.75*yOffset)
		canvas.MoveTo(p1)
		canvas.CurveTo(bcp1, bcp2, p2)
		canvas.text_color = color
		canvas.TextOut(center, "%3.1f"%value)
		
	def drawVerticalSection(self, canvas, color, start, end, height, value, direction=1, lineCount=0, text=None):
		"""Draw the vertical section measurements with cutesy curves and colors."""
		s = min(start, end)
		e = max(start, end)
		start = s
		end = e
		k = (round(start, 2), round(end, 2))
		if k in self._done:
			return
		self._done.append(k)
		if direction == 1:
			canvas.pen_style = cPEN_SOLID
			extraTextShift = 0
		else:
			canvas.pen_style = cPEN_DOT
			extraTextShift = -10
		canvas.brush_color = color
		p1 = fl.GlyphToScreen(Point(height, start))
		p2 = fl.GlyphToScreen(Point(height, end))
		xOffset = int((end-start)*.1)
		d = max(p1.y, p2.y) - min(p1.y, p2.y)
		bcp1 = Point(p1.x+xOffset, int(p1.y-.33*d))
		bcp2 = Point(p1.x+xOffset, int(p2.y+.33*d))

		canvas.MoveTo(p1)
		canvas.CurveTo(bcp1, bcp2, p2)
		canvas.text_color = color
		lineSpace = 7
		center = Point(p1.x + 0.75*xOffset + 5, (p1.y+p2.y)*0.5-10+self.labelOffset[lineCount%len(self.labelOffset)]*lineSpace)
		if text is None:
			text = ""
		canvas.TextOut(center, "%3.1f %s"%(value, text))

	def paint(self, canvas):
		"""
		
			Draw something on the canvas
			
			FL canvas methods:

			[	'Convert', 'CurveTo', 'Ellipse', 'FillGlyph',
			'FitGlyph', 'LineTo', 'MoveTo', 'OutlineGlyph',
			'PutImage', 'Rectangle', 'SplineTo',
			'TextOut', 'UnConvert']
			
		"""
		
		g = CurrentGlyph()
		textYOffset = 8
		yOffset = 0
		self._done = []
		if self.currentGlyphName != g.name:
			self.contourMargins = None
			self.currentGlyphName = g.name
			return
		index = 0
		
		if self.isHorizontal:
			if self.allMargins is not None:
				for j in range(len(self.allMargins)-1):
					index += 1
					x1 = self.allMargins[j]
					x2 = self.allMargins[j+1]
					delta = max(x1,x2)-min(x1, x2)
					x1 = int(round(x1))
					x2 = int(round(x2))
					self.drawHorizontalSection(canvas, self._getColor(index), x1, x2, self.sampleHeight, delta, direction=2)
			
			if self.contourMargins is not None:
				lines = []
				for index, hits in self.contourMargins.items():
					canvas.pen_style = cPEN_SOLID
					#canvas.draw_style = cDS_NOTXOR
					canvas.text_color = 0x000000
					canvas.bk_color = 0xffff00
					for j in range(len(self.contourMargins[index])-1):
						x1 = self.contourMargins[index][j]
						x2 = self.contourMargins[index][j+1]
						delta = max(x1,x2)-min(x1, x2)
						#x1, x2 = self.contourMargins[i]
						x1 = int(round(x1))
						x2 = int(round(x2))
						self.drawHorizontalSection(canvas, self._getColor(index), x1, x2, self.sampleHeight, delta)

			# margins		
			if self.glyphMargins is not None:
				self.drawHorizontalSection(canvas, self._getColor(5), 0, min(self.glyphMargins), self.sampleHeight, min(self.glyphMargins))
				self.drawHorizontalSection(canvas, self._getColor(6), max(self.glyphMargins), g.width, self.sampleHeight, g.width-max(self.glyphMargins))

		else:
			#print "vertival measurement"
			#print "self.allMargins", self.allMargins

			if self.allMargins is not None:
				lineCount = 0
				for j in range(len(self.allMargins)-1):
					lineCount+=1
					index += 1
					x1 = self.allMargins[j]
					x2 = self.allMargins[j+1]
					delta = max(x1,x2)-min(x1, x2)
					x1 = int(round(x1))
					x2 = int(round(x2))
					self.drawVerticalSection(canvas, self._getColor(index), x1, x2, self.sampleHeight, delta, direction=2, lineCount=lineCount)
			
			if self.contourMargins is not None:
				lines = []
				lineCount = 0
				for index, hits in self.contourMargins.items():
					lineCount+=1
					canvas.pen_style = cPEN_SOLID
					#canvas.draw_style = cDS_NOTXOR
					canvas.text_color = 0x000000
					canvas.bk_color = 0xffff00
					for j in range(len(self.contourMargins[index])-1):
						x1 = self.contourMargins[index][j]
						x2 = self.contourMargins[index][j+1]
						delta = max(x1,x2)-min(x1, x2)
						#x1, x2 = self.contourMargins[i]
						x1 = int(round(x1))
						x2 = int(round(x2))
						self.drawVerticalSection(canvas, self._getColor(index), x1, x2, self.sampleHeight, delta, lineCount=lineCount)
						
			# margins		
			if self.glyphMargins is not None:
				f = CurrentFont()
				
				# distance to descender
				delta = max(f.info.descender, min(self.glyphMargins)) - min(f.info.descender, min(self.glyphMargins))
				self.drawVerticalSection(canvas, self._getColor(5), f.info.descender, min(self.glyphMargins),
					self.sampleHeight, delta, text="to descender")

				# distance to ascender
				delta = max(f.info.ascender, max(self.glyphMargins)) - min(f.info.ascender, max(self.glyphMargins))
				self.drawVerticalSection(canvas, self._getColor(6), max(self.glyphMargins), f.info.ascender,
					self.sampleHeight, delta, text="to ascender")

				# distance to baseline
				self.drawVerticalSection(canvas, self._getColor(6), 0, min(self.glyphMargins),
					self.sampleHeight, min(self.glyphMargins), text="to baseline")
			

	def down(self, x, y, keys):
		self.mouseDown = True
		p = Point(x, y)
		p = fl.ScreenToGlyph(p)
		self.modKey = self.keyCallbacks.get(keys)
		self._measure(p)
		
	def _measure(self, p):
		g = CurrentGlyph()
		f = CurrentFont()
		if self.modKey == "mod_cmd":
			self.isHorizontal = False
			self.sampleHeight = p.x
		else:
			self.isHorizontal = True
			self.sampleHeight = p.y
		pen = MarginPen(f, self.sampleHeight, isHorizontal=self.isHorizontal)
		g.draw(pen)
		self.contourMargins = pen.getContourMargins()
		self.glyphMargins = pen.getMargins()
		self.allMargins = pen.getAll()
		g.update()
	
	def up(self, x, y, keys):
		self.mouseDown = False
		self.currentKeys = None
	
	def move(self, x, y, keys):
		if not self.mouseDown:
			return
		p = Point(x, y)
		p = fl.ScreenToGlyph(p)
		self.modKey = self.keyCallbacks.get(keys)
		self._measure(p)
			
	
#t = Tool()
